# -----------------------------------------------------------------------------
# Programm         : C h e c k . p m
# -----------------------------------------------------------------------------
# Autor            : Christian Dhl
# Erstellt         : 12. Juli 2003
# Zuletzt gendert : 22. Juli 2003
# Aufgabe          : Optischer Dinge-Checker :)
# -----------------------------------------------------------------------------

package Check;

#------------------------------------------------------------------------------
# Pakete / Pragmas
#------------------------------------------------------------------------------
use diagnostics;
use strict;
use warnings;

#------------------------------------------------------------------------------
# globale Variablen (modulintern):
#------------------------------------------------------------------------------
my $Debug = 1;             # Wert fr Debug ausgaben, je hher der Wert, desto
                           # mehr wird ausgegeben, bei 0 wird nichts
                           # ausgegeben.
my $Statuszeile;           # Text, der unten im angezeigten Fenster in einem
                           # schmalen Rahmen zu sehen ist.
my $Pfad  = finde_pfad();  # Pfad


{ # auerhalb dieses Blocks kann die $Version nur ber version() gelesen werden.
    my $Version     = '0.0.9';
    my $Datum       = '22.07.2003';

    sub datum {
        return $Datum;
    }
    sub version {
        return $Version;
    }
}


# -----------------------------------------------------------------------------
# sub          : n e w
# -----------------------------------------------------------------------------
# Autor        : Christian Dhl
# Aufgabe      : Erstellt ein neues Check-Objekt.
# Parameter    : Als Hash. Werte:
#                    wm      - Referenz auf MainWindow
#                    check   - Referenz auf AoH mit Text (Titel)   und
#                                                   Test (0, 1, X)
#                    lampen  - 2 (rot/grn) oder 3 (rot/gelb/grn)
#                    fenster - 0 direkt ins mw, 1 in eigenes Toplevel-Window.
#                              3 in zu erzeugendem Frame im MainWindow
#                    button  - 1 = Mit OK-Button, 0 ohne
#                    height  - Hhe in Pixeln
#                    width   - Breite in Pixeln
#                    grab    - 1 = TopLevel-Window grabt den Focus
#                    xpos    - Position des neuen Toplevels in X-Richtung
#                    ypos    - Position des neuen Toplevels in Y-Richtung
#                    title   - Titel des Toplevel-Windows
#                    status  - 1 = Mit Statuszeile, 0 ohne
# Rckgabewert : ge'bless'tes Objekt
# -----------------------------------------------------------------------------
# 0.0.1 - 12.07.2003 - CD - Erstellt
# 0.0.2 - 13.07.2003 - CD - Lampen hinzugefgt
# 0.0.3 - 16.07.2003 - CD - Fenster hinzugefgt
#                           Pfad hinzugefgt
#                           Pfad entfernt (wird ber @INC bestimmt)
# 0.0.4 - 21.07.2003 - CD - button-Behandlung und height und width hinzugefgt
#                           grab hinzugefgt
#                           xpos und ypos hinzugefgt
#                           title hinzugefgt
# 0.0.5 - 22.07.2003 - CD - Behandlung fr Frames (fenster => 2).
#                           status hinzugefgt
# -----------------------------------------------------------------------------
sub new {
    my $invocant = shift;
    my $class = ref($invocant) || $invocant; # Klasse des Aufrufenden Objekts
                                             # oder direkt die Klasse
    # Obiger Code erlaubt: $t2 = $t1->new(); fr Check-Objekt $t1.

    my $self = {mw      => undef, # Referenz auf MainWindow
                check   => undef, # Referenz auf AoH
                lampen  => 2,     # 2 oder 3 Lampen
                fenster => 1,     # Im eigenen Fenster (bei Null wird es ins mw
                                  # gepackt).
                button  => 1,     # Mit Ok-Button.
                height  => 600,   # Hhe bei Toplevelfenster
                width   => 300,   # Breite bei Toplevelfenster
                grab    => 1,     # Toplevelfenster grabt Cursor
                xpos    => undef, # X-Koordinate fr TopLevel-Fenster
                ypos    => undef, # Y-Koordinate fr TopLevel-Fenster
                title   => 'Check V.' . version() . ' (' . datum() . ')',
                status  => 1,
                @_};              # Attribute berschreiben...

    return bless($self, $class);
} # sub new


# -----------------------------------------------------------------------------
# sub          : s h o w
# -----------------------------------------------------------------------------
# Autor        : Christian Dhl
# Aufgabe      : Zeigt das Objekt an.
# Parameter    : keine (Modulhash)
# Rckgabewert : TopLevel
# -----------------------------------------------------------------------------
# 0.0.1 - 12.07.2003 - CD - Erstellt
# 0.0.2 - 13.07.2003 - CD - Lampen hinzugefgt
# 0.0.3 - 16.07.2003 - CD - Fensterbehandlung mw / toplevel hinzugefgt
#                           Pfade gendert
# 0.0.4 - 21.07.2003 - CD - button-Behandlung und height und width hinzugefgt
#                           TopLevel-Widget wird zurckgegeben.
#                           grab hinzugefgt
#                           xpos und ypos hinzugefgt
#                           title hinzugefgt
# 0.0.5 - 22.07.2003 - CD - Behandlung fr Frames (fenster => 2).
#                           status hinzugefgt
#                           Unntze Leerzeile am Ende des Check-Feldes ent-
#                           fernt.
# -----------------------------------------------------------------------------
sub show {
    my $self = shift;


    #--------------------------------------------------------------------------
    # Fehler abfangen:
    #--------------------------------------------------------------------------
    return unless defined $self->{mw}
              and defined $self->{check}
              and defined $self->{lampen}
              and defined $self->{fenster}
              and ($self->{lampen} == 2 or $self->{lampen} == 3)
              ;


    #--------------------------------------------------------------------------
    # Neues Toplevel-Fenster erzeugen:
    #--------------------------------------------------------------------------
    my $tl;
    ### Frame ###
    if ($self->{fenster} == 2) {
        $tl = ${$self->{mw}}->Frame(-border => 0,
                                   )
                             ->pack(-side => 'top',
                                    -expand => 1,
                                    -fill => 'both',
                                   );
    }
    ### TopLevel ###
    elsif ($self->{fenster} == 1) {
        $tl = ${$self->{mw}}->Toplevel();
        $tl->title($self->{title});

        my $icon = $tl->Photo(-file   => "$Pfad/res/check.bmp",
                              -height => 32,
                              -width  => 32,
                             );
        $tl->Icon(-image => $icon);
    }
    ### MainWindow ###
    elsif ($self->{fenster} == 0) {
        $tl = ${$self->{mw}};
    }
    ### Fehler ###
    else {
        die "Falscher Wert fr Fenster!";
    }

    #--------------------------------------------------------------------------
    # Ampelbilder:
    #--------------------------------------------------------------------------
    my $l_g_an  = $tl->Photo(-file   => "$Pfad/res/lgan_k.gif",
                             -height => 32,
                             -width  => 28,
                            );
    my $l_g_aus = $tl->Photo(-file   => "$Pfad/res/lgaus_k.gif",
                             -height => 32,
                             -width  => 28,
                            );
    my $l_r_an  = $tl->Photo(-file   => "$Pfad/res/lran_k.gif",
                             -height => 32,
                             -width  => 28,
                            );
    my $l_r_aus = $tl->Photo(-file   => "$Pfad/res/lraus_k.gif",
                             -height => 32,
                             -width  => 28,
                            );
    my $l_y_an  = $tl->Photo(-file   => "$Pfad/res/lyan_k.gif",
                             -height => 32,
                             -width  => 28,
                            );
    my $l_y_aus = $tl->Photo(-file   => "$Pfad/res/lyaus_k.gif",
                             -height => 32,
                             -width  => 28,
                            );

    #--------------------------------------------------------------------------
    # Status-Bar:
    #--------------------------------------------------------------------------
    my $stat_l;
    if ($self->{status}) {
        $stat_l = $tl->Label(-relief       => 'groove', # ridge
                             -textvariable => \$Statuszeile,
                             -borderwidth  => 2,
                            #-background   => '#d7d7d7',
                            #-state        => 'disabled',
                             -anchor       => 'w',
                            );
    }

    #--------------------------------------------------------------------------
    # Zeilen:
    #--------------------------------------------------------------------------
    my $t = $tl->Scrolled('Text',
                          -relief         => 'groove',
                          -borderwidth    => 2,
                          -background     => 'black',
                          -scrollbars     => 'oe',
                         );

    my $z = 0; # Ein Zhler
    for my $c (@{$self->{check}}) {
        $t->insert('end', "\n") if $z++ > 0;
        my $w;
        if ($self->{lampen} == 2) {
            if ($c->{Test}) {
                $w = $t->Label(-image => $l_g_an,
                               -background => 'black',
                              );
                $t->windowCreate('end', -window => $w);
                $w = $t->Label(-image => $l_r_aus,
                               -background => 'black',
                              );
                $t->windowCreate('end', -window => $w);
            }
            else {
                $w = $t->Label(-image => $l_g_aus,
                               -background => 'black',
                              );
                $t->windowCreate('end', -window => $w);
                $w = $t->Label(-image => $l_r_an,
                               -background => 'black',
                              );
                $t->windowCreate('end', -window => $w);
            }
        }
        else {
            if ($c->{Test} == 1) {
                $w = $t->Label(-image => $l_g_an,
                               -background => 'black',
                              );
                $t->windowCreate('end', -window => $w);
                $w = $t->Label(-image => $l_y_aus,
                               -background => 'black',
                              );
                $t->windowCreate('end', -window => $w);
                $w = $t->Label(-image => $l_r_aus,
                               -background => 'black',
                              );
                $t->windowCreate('end', -window => $w);
            }
            elsif ($c->{Test} == 0) {
                $w = $t->Label(-image => $l_g_aus,
                               -background => 'black',
                              );
                $t->windowCreate('end', -window => $w);
                $w = $t->Label(-image => $l_y_aus,
                               -background => 'black',
                              );
                $t->windowCreate('end', -window => $w);
                $w = $t->Label(-image => $l_r_an,
                               -background => 'black',
                              );
                $t->windowCreate('end', -window => $w);
            }
            else {
                $w = $t->Label(-image => $l_g_aus,
                               -background => 'black',
                              );
                $t->windowCreate('end', -window => $w);
                $w = $t->Label(-image => $l_y_an,
                               -background => 'black',
                              );
                $t->windowCreate('end', -window => $w);
                $w = $t->Label(-image => $l_r_aus,
                               -background => 'black',
                              );
                $t->windowCreate('end', -window => $w);
            }
        }

        $w = $t->Label(-text       => $c->{Text},
                       -background => 'black',
                       -foreground => 'white',
                       -font       => '{Times New Roman} 18 {normal}',
                      );
        $t->windowCreate('end', -window => $w);
    }
    $t->configure(-state => 'disabled'); # verhindert Benutzereingaben


    #--------------------------------------------------------------------------
    # Ok-Button
    #--------------------------------------------------------------------------
    my $ok;
    if ($self->{button}) {
        $ok = $tl->Button(-text        => 'OK',
                          -command     => sub { $tl->destroy() },
                          -default     => 'active',
                          -padx        => 15,
                         );
    }

    #--------------------------------------------------------------------------
    # Elemente packen:
    #--------------------------------------------------------------------------
    if ($self->{status}) {
        $stat_l->pack(-side   => 'bottom',
                     #-expand => 1,
                      -fill   => 'x',
                      -anchor => 'w',
                     );
    }
    if ($self->{button}) {
        $ok->    pack(-expand => '0',
                      -fill   => 'none',
                      -side   => 'bottom',
                     );
    }
    $t->         pack(-side   => 'top',
                      -expand => 1,
                      -fill   => 'both',
                     );

    #--------------------------------------------------------------------------
    # Bindungen:
    #--------------------------------------------------------------------------
    if ($self->{button}) {
        $tl->bind ('<Return>', sub{$ok->invoke()});
        $tl->bind ('<Escape>', sub{$ok->invoke()});
    }

    #--------------------------------------------------------------------------
    # Rest:
    # (grab verhindert das zurckschalten auf Mainwindow per Klick, focus sorgt
    # dafr, dass $tl auch wirklich aktiv ist.)
    #--------------------------------------------------------------------------
    ### TopLevel ###
    if ($self->{fenster} == 1) {
        my $w = 300;
        my $h = 600;
        $w = $self->{width}  if exists  $self->{width} and
                                defined $self->{width};
        $h = $self->{height} if exists  $self->{height} and
                                defined $self->{height};
        if (exists  $self->{xpos} and
            exists  $self->{ypos} and
            defined $self->{xpos} and
            defined $self->{ypos})
        {
            $tl->geometry($w . 'x' . $h . $self->{xpos} . $self->{ypos});
        }
        else {
            $tl->geometry($w . 'x' . $h);
        }
        $tl->grab() if $self->{grab};
        $tl->focus();
    }

    $Statuszeile = 'Willkommen bei Check';

    return $tl;
} # sub show


# -----------------------------------------------------------------------------
# sub          : f i n d e _ p f a d
# -----------------------------------------------------------------------------
# Autor        : Christian Dhl
# Aufgabe      : Sucht den Pfad des Moduls.
# Parameter    : keiner (wertet @INC aus)
# Rckgabewert : Verzeichnis
# -----------------------------------------------------------------------------
# 0.0.1 - 16.07.2003 - CD - Erstellt
# -----------------------------------------------------------------------------
sub finde_pfad {
    for my $pfad (@INC) {
        return $pfad if -e "$pfad/Check.pm";
    }
    return '.';
} # sub finde_pfad

return 1;
